/*
 * board.c
 */

/*
 * 関数名
 *	get_board_status
 * 引数
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		現在のゲームボード
 *	int x
 *		x 座標 ( 0 - BOARD_SIZE )
 *	int y
 *		y 座標 ( 0 - BOARD_SIZE )
 * 戻値
 *	ゲームボードの指定された場所 ( x, y で指定 ) の色 ( BLACK, WHITE, EMPTY )
 * 機能
 *	ゲームボードの指定された場所 ( x, y で指定 ) の色を得る
 *
 */

char get_board_status ( char board[BOARD_SIZE][BOARD_SIZE], int x, int y ) {

  return board[x][y];
}

/*
 * 関数名
 *	set_board_status
 * 引数
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		現在のゲームボード
 *	int x
 *		x 座標 ( 0 - BOARD_SIZE )
 *	int y
 *		y 座標 ( 0 - BOARD_SIZE )
 *	char hand
 *		指定した場所におく色 BLACK, WHITE, EMPTY
 * 戻値
 *	なし (void)
 * 機能
 *	ゲームボードの指定された場所 ( x, y で指定 ) の色を指定した色にする
 *
 */

void set_board_status ( char board[BOARD_SIZE][BOARD_SIZE], int x, int y, char hand ) {
  board[x][y] = hand;
}

/*
 * 関数名
 *	init_board
 * 引数
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		現在のゲームボード
 * 戻値
 *	なし (void)
 * 機能
 *	ゲームボードを初期状態にする
 *
 */

void init_board ( char board[BOARD_SIZE][BOARD_SIZE] ) {
  int x;
  int y;

  for ( y = 0; y < BOARD_SIZE; y++ ) {
    for ( x = 0; x < BOARD_SIZE; x++ ) {
      set_board_status ( board, x, y, EMPTY );
    }
  }

  set_board_status ( board, 3, 4, WHITE );
  set_board_status ( board, 4, 3, WHITE );
  set_board_status ( board, 3, 3, BLACK );
  set_board_status ( board, 4, 4, BLACK );

}

/*
 * 関数名
 *	print_board
 * 引数
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		現在のゲームボード
 * 戻値
 *	なし (void)
 * 機能
 *	ゲームボードを表示する
 */

void print_board ( char board[BOARD_SIZE][BOARD_SIZE] ) {
  int x;
  int y;

  printf ( "  " );
  for ( x = 0; x < BOARD_SIZE; x++ ) {
    printf ( " %c ", 'A' + x );
  }

  printf ( "\n" );

  for ( y = 0; y < BOARD_SIZE; y++ ) {
    printf ( " %d", y + 1 );
    for ( x = 0; x < BOARD_SIZE; x++ ) {
	  if ( get_board_status ( board, x, y ) == BLACK ) {
		printf ( "◯"  );
	  } else if ( get_board_status ( board, x, y ) == WHITE ) {
		printf ( "●"  );
	  } else {
		printf ( "  "  );
	  }
    }
    printf ( "\n" );
  }
}

/*
 * 関数名
 *	z_inner_board
 * 引数
 *	int z
 *		座標 ( x, y 問わず )
 * 戻値
 *	はいっていれば 1 そうでなければ 0
 * 機能
 *	指定した座標が、ボード内部にあるかどうかを判定
 */

int z_inner_board ( int z ) {

  return 0 <= z && z < BOARD_SIZE;
}

/*
 * 関数名
 *	x_inner_board
 * 引数
 *	int x
 *		x 座標
 * 戻値
 *	はいっていれば 1 そうでなければ 0
 * 機能
 *	指定した x 座標が、ボード内部にあるかどうかを判定
 */

int x_inner_board ( int x ) {

  return z_inner_board ( x );
}

/*
 * 関数名
 *	y_inner_board
 * 引数
 *	int y
 *		y 座標
 * 戻値
 *	はいっていれば 1 そうでなければ 0
 * 機能
 *	指定した y 座標が、ボード内部にあるかどうかを判定
 */

int y_inner_board (int y ) {

  return z_inner_board ( y );
}

/*
 * 関数名
 *	inner_board
 * 引数
 *	int x
 *		x 座標
 *	int y
 *		y 座標
 * 戻値
 *	はいっていれば 1 そうでなければ 0
 * 機能
 *	指定した座標 (x, y) が、ボード内部にあるかどうかを判定
 */

int inner_board ( int x, int y ) {

  return x_inner_board ( x ) && y_inner_board ( y );
}

/*
 * 関数名
 *	get_oposit_color
 * 引数
 *	char color
 *		現在の色
 * 戻値
 *	対戦相手の色
 * 機能
 *	現在の色から、対戦相手の色を知る 
 */

int get_oposit_color ( int color ) {

  return WHITE + BLACK - color;

}

