/*
 * rule.c
 */

/*
 * ֐
 *	playable
 * 
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		݂̃Q[{[h
 *	int x
 *		x W ( 0 - BOARD_SIZE )
 *	int y
 *		y W ( 0 - BOARD_SIZE )
 *	char color
 *		݂̐F
 * ߒl
 *	int
 *		w肵ꏊɁA̐F̋Ȃ YES
 *		w肵ꏊɁA̐F̋ȂȂ NO
 *		G[ ERROR
 * @\
 *	w肵ꏊɋ邩ǂ̔
 */

int playable ( char board[BOARD_SIZE][BOARD_SIZE], int x, int y, char color ) {
  int dir;

  if ( get_board_status ( board, x, y ) != EMPTY ) {
    return ERROR;
  }

  for ( dir = DIRECT_MIN; dir <= DIRECT_MAX; dir++ ) {
    if ( check_reversible ( board, x, y, color, dir ) ) {
      return YES;
    }
  }

  return NO;
}

/*
 * ֐
 *	check_reversible
 * 
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		݂̃Q[{[h
 *	int x
 *		x W ( 0 - BOARD_SIZE )
 *	int y
 *		y W ( 0 - BOARD_SIZE )
 *	char color
 *		݂̐F
 *	int dir
 *		
 * ߒl
 *	int
 *		w肵ꏊ̎w肵F̋Ŏw肵ɗԂ̋ YES
 *		łȂ NO
 * @\
 *	w肵ꏊ̎w肵ɗԂ邩ǂ̔
 */

int check_reversible ( char board[BOARD_SIZE][BOARD_SIZE], int x, int y, char color, int dir ) {
    int dx = x_step ( dir );
    int dy = y_step ( dir );
    char oc = get_oposit_color ( color );

    x = x + dx;
    y = y + dy;

    if ( !inner_board ( x, y ) ) {
      return NO;
    }

    if ( get_board_status ( board, x, y ) != oc ) {
      //      printf ( "%d, %d, %c, %c %c\n", x, y, get_board_status ( board, x, y ), oc, color );
      return NO;
    }

    x = x + dx;
    y = y + dy;

    while ( inner_board ( x, y ) ) {
      char st = get_board_status ( board, x, y );

      if ( st == color ) {
	return YES;
      }

      if ( st == EMPTY ) {
	return NO;
      }

      x = x + dx;
      y = y + dy;

    }

    return NO;
}

/*
 * ֐
 *	play_hand
 * 
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		݂̃Q[{[h
 *	int x
 *		x W ( 0 - BOARD_SIZE )
 *	int y
 *		y W ( 0 - BOARD_SIZE )
 *	char color
 *		݂̐F
 * ߒl
 *	void
 *		Ȃ
 * @\
 *	w肵ꏊɋ
 */

void play_hand ( char board[BOARD_SIZE][BOARD_SIZE], int x, int y, char color ) {
  int dir;

  set_board_status ( board, x, y, color );

  for ( dir = DIRECT_MIN; dir <= DIRECT_MAX; dir++ ) {
    if ( check_reversible ( board, x, y, color, dir ) ) {
      do_reverse ( board, x, y, color, dir );
    }
  }
}

/*
 * ֐
 *	do_reverse
 * 
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		݂̃Q[{[h
 *	int x
 *		x W ( 0 - BOARD_SIZE )
 *	int y
 *		y W ( 0 - BOARD_SIZE )
 *	char color
 *		݂̐F
 *	int dir
 *		
 * ߒl
 *	void
 *		Ȃ
 * @\
 *	w肵ꏊAw肵ɗԂs
 */

void do_reverse ( char board[BOARD_SIZE][BOARD_SIZE], int x, int y, char color, int dir ) {
    int dx = x_step ( dir );
    int dy = y_step ( dir );
    char oc = get_oposit_color ( color );

    x = x + dx;
    y = y + dy;

    while ( inner_board ( x, y ) ) {

      if ( get_board_status ( board, x, y ) != oc ) {
	break;
      }

      set_board_status ( board, x, y, color );

      x = x + dx;
      y = y + dy;
    }
}

/*
 * ֐
 *	check_playable_color
 * 
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		݂̃Q[{[h
 *	char color
 *		݂̐F
 * ߒl
 *	int
 *		̐F̃vC[͋łĂ YES łȂ NO
 * @\
 *	̐F̃vC[͋łĂ邩ǂ𔻒
 */

int check_playable_color ( char board[BOARD_SIZE][BOARD_SIZE], char color ) {
  int x;
  int y;

  for ( y = 0; y < BOARD_SIZE; y++ ) {
    for ( x = 0; x < BOARD_SIZE; x++ ) {
      if ( playable ( board, x, y, color ) ) {
	return YES;
      }
    }
  }

  return NO;
}

/*
 * ֐
 *	check_end
 * 
 *	char board[BOARD_SIZE][BOARD_SIZE]
 *		݂̃Q[{[h
 * ߒl
 *	int
 *		łĂЂƂȂ YES łȂ NO
 * @\
 *	I𔻒
 */

int check_end ( char board[BOARD_SIZE][BOARD_SIZE] ) {

  return !check_playable_color ( board, WHITE ) && !check_playable_color ( board, BLACK ); 
}
